# Copyright 2014 eBay Software Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from trove.common import cfg
from trove.common.strategies.cluster import strategy
from trove.common.views import create_links
from trove.instance.views import InstanceDetailView

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class ClusterView(object):

    def __init__(self, cluster, req=None, load_servers=True):
        self.cluster = cluster
        self.req = req
        self.load_servers = load_servers

    def data(self):
        instances, ip_list = self.build_instances()
        cluster_dict = {
            "id": self.cluster.id,
            "name": self.cluster.name,
            "task": {"id": self.cluster.task_id,
                     "name": self.cluster.task_name,
                     "description": self.cluster.task_description},
            "created": self.cluster.created,
            "updated": self.cluster.updated,
            "links": self._build_links(),
            "datastore": {"type": self.cluster.datastore.name,
                          "version": self.cluster.datastore_version.name},
            "instances": instances
        }
        if ip_list:
            cluster_dict["ip"] = ip_list

        extended_properties = self.get_extended_properties()
        if extended_properties:
            cluster_dict["extended_properties"] = extended_properties

        if self.cluster.locality:
            cluster_dict['locality'] = self.cluster.locality
        if self.cluster.configuration_id:
            cluster_dict['configuration'] = self.cluster.configuration_id
        LOG.debug(cluster_dict)
        return {"cluster": cluster_dict}

    def _build_links(self):
        return create_links("clusters", self.req, self.cluster.id)

    def _build_instances(self, ip_to_be_published_for=[],
                         instance_dict_to_be_published_for=[]):
        instances = []
        ip_list = []
        if self.load_servers:
            cluster_instances = self.cluster.instances
        else:
            cluster_instances = self.cluster.instances_without_server
        for instance in cluster_instances:
            instance_dict = {
                "id": instance.id,
                "name": instance.name,
                "type": instance.type,
                "links": create_links("instances", self.req, instance.id)
            }
            if instance.shard_id:
                instance_dict["shard_id"] = instance.shard_id
            if self.load_servers:
                instance_dict["status"] = instance.status
                if CONF.get(instance.datastore_version.manager).volume_support:
                    instance_dict["volume"] = {"size": instance.volume_size}
                instance_dict["flavor"] = self._build_flavor_info(
                    instance.flavor_id)
            instance_ips = instance.get_visible_ip_addresses()
            if self.load_servers and instance_ips:
                instance_dict["ip"] = instance_ips
                if instance.type in ip_to_be_published_for:
                    ip_list.extend(instance_ips)
            if instance.type in instance_dict_to_be_published_for:
                instances.append(instance_dict)
        return instances, ip_list

    def build_instances(self):
        raise NotImplementedError()

    def get_extended_properties(self):
        return None

    def _build_flavor_info(self, flavor_id):
        return {
            "id": flavor_id,
            "links": create_links("flavors", self.req, flavor_id)
        }


class ClusterInstanceDetailView(InstanceDetailView):
    def __init__(self, instance, req):
        super(ClusterInstanceDetailView, self).__init__(instance, req=req)

    def data(self):
        result = super(ClusterInstanceDetailView, self).data()
        return result


class ClustersView(object):
    def __init__(self, clusters, req=None):
        self.clusters = clusters
        self.req = req

    def data(self):
        data = []
        for cluster in self.clusters:
            data.append(self.data_for_cluster(cluster))
        return {'clusters': data}

    def data_for_cluster(self, cluster):
        view = load_view(cluster, req=self.req, load_servers=False)
        return view.data()['cluster']


def load_view(cluster, req, load_servers=True):
    manager = cluster.datastore_version.manager
    return strategy.load_api_strategy(manager).cluster_view_class(
        cluster, req, load_servers)



# Copyright 2013 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import os
import time

from proboscis import asserts
from proboscis.decorators import time_out
from proboscis import SkipTest
from proboscis import test
from troveclient.compat import exceptions

from trove import tests
from trove.tests.api.instances import instance_info
from trove.tests.config import CONFIG
from trove.tests.api import configurations


def do_not_delete_instance():
    return os.environ.get("TESTS_DO_NOT_DELETE_INSTANCE", None) is not None


@test(depends_on_groups=[tests.DBAAS_API_REPLICATION],
      groups=[tests.DBAAS_API_INSTANCES_DELETE],
      enabled=not do_not_delete_instance())
class TestDeleteInstance(object):
    @time_out(3 * 60)
    @test
    def test_delete(self):
        """Delete instance for clean up."""
        if not hasattr(instance_info, "initial_result"):
            raise SkipTest("Instance was never created, skipping test...")
        # Update the report so the logs inside the instance will be saved.
        CONFIG.get_report().update()

        dbaas = instance_info.dbaas
        dbaas.instances.delete(instance_info.id)

        attempts = 0
        try:
            time.sleep(1)
            result = True
            while result is not None:
                attempts += 1
                result = dbaas.instances.get(instance_info.id)
                asserts.assert_equal(200, dbaas.last_http_code)
                asserts.assert_equal("SHUTDOWN", result.status)
                time.sleep(1)
        except exceptions.NotFound:
            pass
        except Exception as ex:
            asserts.fail("A failure occurred when trying to GET instance %s "
                         "for the %d time: %s" %
                         (str(instance_info.id), attempts, str(ex)))

    @test(depends_on=[test_delete])
    def test_instance_status_deleted_in_db(self):
        """test_instance_status_deleted_in_db"""
        dbaas_admin = instance_info.dbaas_admin
        mgmt_details = dbaas_admin.management.index(deleted=True)
        for instance in mgmt_details:
            if instance.id == instance_info.id:
                asserts.assert_equal(instance.service_status, 'DELETED')
                break
        else:
            asserts.fail("Could not find instance %s" % instance_info.id)

    @test(depends_on=[test_instance_status_deleted_in_db])
    def test_delete_datastore(self):
        dbaas_admin = instance_info.dbaas_admin

        datastore = dbaas_admin.datastores.get(
            CONFIG.dbaas_datastore_name_no_versions)
        versions = dbaas_admin.datastore_versions.list(datastore.id)
        for version in versions:
            dbaas_admin.mgmt_datastore_versions.delete(version.id)

        # Delete the datastore
        dbaas_admin.datastores.delete(datastore.id)

    @test(depends_on=[test_instance_status_deleted_in_db])
    def test_delete_configuration(self):
        """Delete configurations created during testing."""
        dbaas_admin = instance_info.dbaas_admin
        configs = dbaas_admin.configurations.list()
        for config in configs:
            if config.name == configurations.CONFIG_NAME:
                dbaas_admin.configurations.delete(config.id)



# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import time

from proboscis import after_class
from proboscis import asserts
from proboscis import before_class
from proboscis.decorators import time_out
from proboscis import SkipTest
from proboscis import test
from troveclient.compat.exceptions import BadRequest
from troveclient.compat.exceptions import HTTPNotImplemented

from trove.common import cfg
from trove.common.utils import poll_until
from trove import tests
from trove.tests.api.instances import assert_unprocessable
from trove.tests.api.instances import EPHEMERAL_SUPPORT
from trove.tests.api.instances import instance_info
from trove.tests.api.instances import VOLUME_SUPPORT
from trove.tests.config import CONFIG
import trove.tests.util as testsutil
from trove.tests.util.check import TypeCheck
from trove.tests.util import LocalSqlClient
from trove.tests.util.server_connection import create_server_connection

MYSQL_USERNAME = "test_user"
MYSQL_PASSWORD = "abcde"
FAKE_MODE = CONFIG.fake_mode
# If true, then we will actually log into the database.
USE_IP = not FAKE_MODE


class MySqlConnection(object):

    def __init__(self, host):
        self.host = host

    def connect(self):
        """Connect to MySQL database."""
        print("Connecting to MySQL, mysql --host %s -u %s -p%s"
              % (self.host, MYSQL_USERNAME, MYSQL_PASSWORD))
        sql_engine = LocalSqlClient.init_engine(MYSQL_USERNAME, MYSQL_PASSWORD,
                                                self.host)
        self.client = LocalSqlClient(sql_engine, use_flush=False)

    def is_connected(self):
        cmd = "SELECT 1;"
        try:
            with self.client:
                self.client.execute(cmd)
            return True
        except Exception as e:
            print(
                "Failed to execute command: %s, error: %s" % (cmd, str(e))
            )
            return False

    def execute(self, cmd):
        try:
            with self.client:
                self.client.execute(cmd)
            return True
        except Exception as e:
            print(
                "Failed to execute command: %s, error: %s" % (cmd, str(e))
            )
            return False


# Use default value from trove.common.cfg, and it could be overridden by
# a environment variable when the tests run.
def get_resize_timeout():
    value_from_env = os.environ.get("TROVE_RESIZE_TIME_OUT", None)
    if value_from_env:
        return int(value_from_env)

    return cfg.CONF.resize_time_out


TIME_OUT_TIME = get_resize_timeout()


class ActionTestBase(object):
    """Has some helpful functions for testing actions.

    The test user must be created for some of these functions to work.

    """

    def set_up(self):
        """If you're using this as a base class, call this method first."""
        self.dbaas = instance_info.dbaas
        if USE_IP:
            address = instance_info.get_address()
            self.connection = MySqlConnection(address)

    @property
    def instance(self):
        return self.dbaas.instances.get(self.instance_id)

    @property
    def instance_address(self):
        return instance_info.get_address()

    @property
    def instance_mgmt_address(self):
        return instance_info.get_address(mgmt=True)

    @property
    def instance_id(self):
        return instance_info.id

    def create_user(self):
        """Create a MySQL user we can use for this test."""

        users = [{"name": MYSQL_USERNAME, "password": MYSQL_PASSWORD,
                  "databases": [{"name": MYSQL_USERNAME}]}]
        self.dbaas.users.create(instance_info.id, users)

        def has_user():
            users = self.dbaas.users.list(instance_info.id)
            return any([user.name == MYSQL_USERNAME for user in users])

        poll_until(has_user, time_out=30)
        if not FAKE_MODE:
            time.sleep(5)

    def ensure_mysql_is_running(self):
        if USE_IP:
            self.connection.connect()
            asserts.assert_true(self.connection.is_connected(),
                                "Unable to connect to MySQL.")

            self.proc_id = self.find_mysql_proc_on_instance()
            asserts.assert_is_not_none(self.proc_id,
                                       "MySQL process can not be found.")

        asserts.assert_is_not_none(self.instance)
        asserts.assert_true(self.instance.status in CONFIG.running_status)

    def find_mysql_proc_on_instance(self):
        server = create_server_connection(
            self.instance_id,
            ip_address=self.instance_mgmt_address
        )
        container_exist_cmd = 'sudo docker ps -q'
        pid_cmd = "sudo docker inspect database -f '{{.State.Pid}}'"

        try:
            server.execute(container_exist_cmd)
        except Exception as err:
            asserts.fail("Failed to execute command: %s, error: %s" %
                         (container_exist_cmd, str(err)))

        try:
            stdout = server.execute(pid_cmd)
            return int(stdout)
        except ValueError:
            return None
        except Exception as err:
            asserts.fail("Failed to execute command: %s, error: %s" %
                         (pid_cmd, str(err)))

    def log_current_users(self):
        users = self.dbaas.users.list(self.instance_id)
        CONFIG.get_report().log("Current user count = %d" % len(users))
        for user in users:
            CONFIG.get_report().log("\t" + str(user))

    def _build_expected_msg(self):
        expected = {
            'instance_size': instance_info.dbaas_flavor.ram,
            'tenant_id': instance_info.user.tenant_id,
            'instance_id': instance_info.id,
            'instance_name': instance_info.name,
            'created_at': testsutil.iso_time(
                instance_info.initial_result.created),
            'launched_at': testsutil.iso_time(self.instance.updated),
            'modify_at': testsutil.iso_time(self.instance.updated)
        }
        return expected


@test(depends_on_groups=[tests.DBAAS_API_INSTANCES])
def create_user():
    """Create a test user so that subsequent tests can log in."""
    helper = ActionTestBase()
    helper.set_up()
    if USE_IP:
        try:
            helper.create_user()
        except BadRequest:
            pass  # Ignore this if the user already exists.
        helper.connection.connect()
        asserts.assert_true(helper.connection.is_connected(),
                            "Test user must be able to connect to MySQL.")


class RebootTestBase(ActionTestBase):
    """Tests restarting MySQL."""

    def call_reboot(self):
        raise NotImplementedError()

    def wait_for_successful_restart(self):
        """Wait until status becomes running.

        Reboot is an async operation, make sure the instance is rebooting
        before active.
        """
        def _is_rebooting():
            instance = self.instance
            if instance.status == "REBOOT":
                return True
            return False

        poll_until(_is_rebooting, time_out=TIME_OUT_TIME)

        def is_finished_rebooting():
            instance = self.instance
            asserts.assert_not_equal(instance.status, "ERROR")
            if instance.status in CONFIG.running_status:
                return True
            return False

        poll_until(is_finished_rebooting, time_out=TIME_OUT_TIME)

    def assert_mysql_proc_is_different(self):
        if not USE_IP:
            return
        new_proc_id = self.find_mysql_proc_on_instance()
        asserts.assert_not_equal(new_proc_id, self.proc_id,
                                 "MySQL process ID should be different!")

    def successful_restart(self):
        """Restart MySQL via the REST API successfully."""
        self.call_reboot()
        self.wait_for_successful_restart()
        self.assert_mysql_proc_is_different()

    def wait_for_failure_status(self):
        """Wait until status becomes running."""
        def is_finished_rebooting():
            instance = self.instance
            if instance.status in ['REBOOT', 'ACTIVE', 'HEALTHY']:
                return False
            # The reason we check for BLOCKED as well as SHUTDOWN is because
            # Upstart might try to bring mysql back up after the borked
            # connection and the guest status can be either
            asserts.assert_true(instance.status in ("SHUTDOWN", "BLOCKED"))
            return True

        poll_until(is_finished_rebooting, time_out=TIME_OUT_TIME)

    def wait_for_status(self, status, timeout=60, sleep_time=5):
        def is_status():
            instance = self.instance
            if instance.status in status:
                return True
            return False

        poll_until(is_status, time_out=timeout, sleep_time=sleep_time)

    def wait_for_operating_status(self, status, timeout=60, sleep_time=5):
        def is_status():
            instance = self.instance
            if instance.operating_status in status:
                return True
            return False

        poll_until(is_status, time_out=timeout, sleep_time=sleep_time)


@test(groups=[tests.DBAAS_API_INSTANCE_ACTIONS],
      depends_on_groups=[tests.DBAAS_API_DATABASES],
      depends_on=[create_user])
class RestartTests(RebootTestBase):
    """Test restarting MySQL."""

    def call_reboot(self):
        self.instance.restart()
        asserts.assert_equal(202, self.dbaas.last_http_code)

    @before_class
    def test_set_up(self):
        self.set_up()

    @test
    def test_ensure_mysql_is_running(self):
        """Make sure MySQL is accessible before restarting."""
        self.ensure_mysql_is_running()

    @test(depends_on=[test_ensure_mysql_is_running])
    def test_successful_restart(self):
        """Restart MySQL via the REST API successfully."""
        self.successful_restart()


@test(groups=[tests.DBAAS_API_INSTANCE_ACTIONS],
      depends_on_classes=[RestartTests])
class StopTests(RebootTestBase):
    """Test stopping MySQL."""

    def call_reboot(self):
        self.instance.restart()

    @before_class
    def test_set_up(self):
        self.set_up()

    @test
    def test_ensure_mysql_is_running(self):
        """Make sure MySQL is accessible before restarting."""
        self.ensure_mysql_is_running()

    @test(depends_on=[test_ensure_mysql_is_running])
    def test_stop_mysql(self):
        """Stops MySQL by admin."""
        instance_info.dbaas_admin.management.stop(self.instance_id)
        self.wait_for_operating_status(['SHUTDOWN'], timeout=90, sleep_time=10)

    @test(depends_on=[test_stop_mysql])
    def test_volume_info_while_mysql_is_down(self):
        """
        Confirms the get call behaves appropriately while an instance is
        down.
        """
        if not VOLUME_SUPPORT:
            raise SkipTest("Not testing volumes.")
        instance = self.dbaas.instances.get(self.instance_id)
        with TypeCheck("instance", instance) as check:
            check.has_field("volume", dict)
            check.true('size' in instance.volume)
            check.true('used' in instance.volume)
            check.true(isinstance(instance.volume.get('size', None), int))
            check.true(isinstance(instance.volume.get('used', None), float))

    @test(depends_on=[test_volume_info_while_mysql_is_down])
    def test_successful_restart_from_shutdown(self):
        """Restart MySQL via the REST API successfully when MySQL is down."""
        self.successful_restart()


@test(groups=[tests.DBAAS_API_INSTANCE_ACTIONS],
      depends_on_classes=[StopTests])
class RebootTests(RebootTestBase):
    """Test restarting instance."""

    def call_reboot(self):
        instance_info.dbaas_admin.management.reboot(self.instance_id)

    @before_class
    def test_set_up(self):
        self.set_up()
        asserts.assert_true(hasattr(self, 'dbaas'))
        asserts.assert_true(self.dbaas is not None)

    @test
    def test_ensure_mysql_is_running(self):
        """Make sure MySQL is accessible before rebooting."""
        self.ensure_mysql_is_running()

    @after_class(depends_on=[test_ensure_mysql_is_running])
    def test_successful_reboot(self):
        """MySQL process is different after rebooting."""
        if FAKE_MODE:
            raise SkipTest("Cannot run this in fake mode.")
        self.successful_restart()


@test(groups=[tests.DBAAS_API_INSTANCE_ACTIONS],
      depends_on_classes=[RebootTests])
class ResizeInstanceTest(ActionTestBase):
    """Test resizing instance."""
    @property
    def flavor_id(self):
        return instance_info.dbaas_flavor_href

    def wait_for_resize(self):
        def is_finished_resizing():
            instance = self.instance
            if instance.status == "RESIZE":
                return False
            asserts.assert_true(instance.status in CONFIG.running_status)
            return True

        poll_until(is_finished_resizing, time_out=TIME_OUT_TIME)

    @before_class
    def setup(self):
        self.set_up()
        if USE_IP:
            self.connection.connect()
            asserts.assert_true(self.connection.is_connected(),
                                "Should be able to connect before resize.")

    @test
    def test_instance_resize_same_size_should_fail(self):
        asserts.assert_raises(BadRequest, self.dbaas.instances.resize_instance,
                              self.instance_id, self.flavor_id)

    @test(enabled=VOLUME_SUPPORT)
    def test_instance_resize_to_ephemeral_in_volume_support_should_fail(self):
        flavor_name = CONFIG.values.get('instance_bigger_eph_flavor_name',
                                        'eph.rd-smaller')
        flavor_id = None
        for item in instance_info.flavors:
            if item.name == flavor_name:
                flavor_id = item.id

        asserts.assert_is_not_none(flavor_id)

        def is_active():
            return self.instance.status in CONFIG.running_status

        poll_until(is_active, time_out=TIME_OUT_TIME)
        asserts.assert_true(self.instance.status in CONFIG.running_status)

        asserts.assert_raises(HTTPNotImplemented,
                              self.dbaas.instances.resize_instance,
                              self.instance_id, flavor_id)

    @test(enabled=EPHEMERAL_SUPPORT)
    def test_instance_resize_to_non_ephemeral_flavor_should_fail(self):
        flavor_name = CONFIG.values.get('instance_bigger_flavor_name',
                                        'm1-small')
        flavor_id = None
        for item in instance_info.flavors:
            if item.name == flavor_name:
                flavor_id = item.id

        asserts.assert_is_not_none(flavor_id)
        asserts.assert_raises(BadRequest, self.dbaas.instances.resize_instance,
                              self.instance_id, flavor_id)

    def obtain_flavor_ids(self):
        old_id = self.instance.flavor['id']
        self.expected_old_flavor_id = old_id
        if EPHEMERAL_SUPPORT:
            flavor_name = CONFIG.values.get('instance_bigger_eph_flavor_name',
                                            'eph.rd-smaller')
        else:
            flavor_name = CONFIG.values.get('instance_bigger_flavor_name',
                                            'm1.small')

        new_flavor = None
        for item in instance_info.flavors:
            if item.name == flavor_name:
                new_flavor = item
                break

        asserts.assert_is_not_none(new_flavor)

        self.old_dbaas_flavor = instance_info.dbaas_flavor
        instance_info.dbaas_flavor = new_flavor
        self.expected_new_flavor_id = new_flavor.id

    @test(depends_on=[test_instance_resize_same_size_should_fail])
    def test_status_changed_to_resize(self):
        """test_status_changed_to_resize"""
        self.log_current_users()
        self.obtain_flavor_ids()
        self.dbaas.instances.resize_instance(
            self.instance_id,
            self.expected_new_flavor_id)
        asserts.assert_equal(202, self.dbaas.last_http_code)

        # (WARNING) IF THE RESIZE IS WAY TOO FAST THIS WILL FAIL
        assert_unprocessable(
            self.dbaas.instances.resize_instance,
            self.instance_id,
            self.expected_new_flavor_id)

    @test(depends_on=[test_status_changed_to_resize])
    @time_out(TIME_OUT_TIME)
    def test_instance_returns_to_active_after_resize(self):
        """test_instance_returns_to_active_after_resize"""
        self.wait_for_resize()

    @test(depends_on=[test_instance_returns_to_active_after_resize,
                      test_status_changed_to_resize])
    def test_resize_instance_usage_event_sent(self):
        expected = self._build_expected_msg()
        expected['old_instance_size'] = self.old_dbaas_flavor.ram
        instance_info.consumer.check_message(instance_info.id,
                                             'trove.instance.modify_flavor',
                                             **expected)

    @test(depends_on=[test_instance_returns_to_active_after_resize],
          runs_after=[test_resize_instance_usage_event_sent])
    def resize_should_not_delete_users(self):
        """Resize should not delete users."""
        # Resize has an incredibly weird bug where users are deleted after
        # a resize. The code below is an attempt to catch this while proceeding
        # with the rest of the test (note the use of runs_after).
        if USE_IP:
            users = self.dbaas.users.list(self.instance_id)
            usernames = [user.name for user in users]
            if MYSQL_USERNAME not in usernames:
                self.create_user()
                asserts.fail("Resize made the test user disappear.")

    @test(depends_on=[test_instance_returns_to_active_after_resize],
          runs_after=[resize_should_not_delete_users])
    def test_make_sure_mysql_is_running_after_resize(self):
        self.ensure_mysql_is_running()

    @test(depends_on=[test_make_sure_mysql_is_running_after_resize])
    def test_instance_has_new_flavor_after_resize(self):
        actual = self.instance.flavor['id']
        asserts.assert_equal(actual, self.expected_new_flavor_id)


@test(depends_on_classes=[ResizeInstanceTest],
      groups=[tests.DBAAS_API_INSTANCE_ACTIONS],
      enabled=VOLUME_SUPPORT)
class ResizeInstanceVolumeTest(ActionTestBase):
    """Resize the volume of the instance."""
    @before_class
    def setUp(self):
        self.set_up()
        self.old_volume_size = int(instance_info.volume['size'])
        self.new_volume_size = self.old_volume_size + 1
        self.old_volume_fs_size = instance_info.get_volume_filesystem_size()

        # Create some databases to check they still exist after the resize
        self.expected_dbs = ['salmon', 'halibut']
        databases = []
        for name in self.expected_dbs:
            databases.append({"name": name})
        instance_info.dbaas.databases.create(instance_info.id, databases)

    @test
    @time_out(60)
    def test_volume_resize(self):
        """test_volume_resize"""
        instance_info.dbaas.instances.resize_volume(instance_info.id,
                                                    self.new_volume_size)

    @test(depends_on=[test_volume_resize])
    def test_volume_resize_success(self):
        """test_volume_resize_success"""

        def check_resize_status():
            instance = instance_info.dbaas.instances.get(instance_info.id)
            if instance.status in CONFIG.running_status:
                return True
            elif instance.status in ["RESIZE", "SHUTDOWN"]:
                return False
            else:
                asserts.fail("Status should not be %s" % instance.status)

        poll_until(check_resize_status, sleep_time=5, time_out=300,
                   initial_delay=5)
        instance = instance_info.dbaas.instances.get(instance_info.id)
        asserts.assert_equal(instance.volume['size'], self.new_volume_size)

    @test(depends_on=[test_volume_resize_success])
    def test_volume_filesystem_resize_success(self):
        """test_volume_filesystem_resize_success"""
        # The get_volume_filesystem_size is a mgmt call through the guestagent
        # and the volume resize occurs through the fake nova-volume.
        # Currently the guestagent fakes don't have access to the nova fakes so
        # it doesn't know that a volume resize happened and to what size so
        # we can't fake the filesystem size.
        if FAKE_MODE:
            raise SkipTest("Cannot run this in fake mode.")
        new_volume_fs_size = instance_info.get_volume_filesystem_size()
        asserts.assert_true(self.old_volume_fs_size < new_volume_fs_size)
        # The total filesystem size is not going to be exactly the same size of
        # cinder volume but it should round to it. (e.g. round(1.9) == 2)
        asserts.assert_equal(round(new_volume_fs_size), self.new_volume_size)

    @test(depends_on=[test_volume_resize_success])
    def test_resize_volume_usage_event_sent(self):
        """test_resize_volume_usage_event_sent"""
        expected = self._build_expected_msg()
        expected['volume_size'] = self.new_volume_size
        expected['old_volume_size'] = self.old_volume_size
        instance_info.consumer.check_message(instance_info.id,
                                             'trove.instance.modify_volume',
                                             **expected)

    @test(depends_on=[test_volume_resize_success])
    def test_volume_resize_success_databases(self):
        """test_volume_resize_success_databases"""
        databases = instance_info.dbaas.databases.list(instance_info.id)
        db_list = []
        for database in databases:
            db_list.append(database.name)
        for name in self.expected_dbs:
            if name not in db_list:
                asserts.fail(
                    "Database %s was not found after the volume resize. "
                    "Returned list: %s" % (name, databases))



#!/usr/bin/env python

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2010 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Installation script for Trove's development virtualenv
"""

import os
import subprocess
import sys


ROOT = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
VENV = os.path.join(ROOT, '.venv')
PIP_REQUIRES = os.path.join(ROOT, 'requirements.txt')
TEST_REQUIRES = os.path.join(ROOT, 'test-requirements.txt')
PY_VERSION = "python%s.%s" % (sys.version_info[0], sys.version_info[1])


def die(message, *args):
    print(message % args, file=sys.stderr)
    sys.exit(1)


def check_python_version():
    if sys.version_info < (2, 7):
        die("Need Python Version >= 2.7")


def run_command(cmd, redirect_output=True, check_exit_code=True):
    """
    Runs a command in an out-of-process shell, returning the
    output of that command.  Working directory is ROOT.
    """
    if redirect_output:
        stdout = subprocess.PIPE
    else:
        stdout = None

    proc = subprocess.Popen(cmd, cwd=ROOT, stdout=stdout)
    output = proc.communicate()[0]
    if check_exit_code and proc.returncode != 0:
        die('Command "%s" failed.\n%s', ' '.join(cmd), output)
    return output


HAS_EASY_INSTALL = bool(run_command(['which', 'easy_install'],
    check_exit_code=False).strip())
HAS_VIRTUALENV = bool(run_command(['which', 'virtualenv'],
    check_exit_code=False).strip())


def check_dependencies():
    """Make sure virtualenv is in the path."""

    if not HAS_VIRTUALENV:
        print('not found.')
        # Try installing it via easy_install...
        if HAS_EASY_INSTALL:
            print('Installing virtualenv via easy_install...'),
            if not (run_command(['which', 'easy_install']) and
                    run_command(['easy_install', 'virtualenv'])):
                die('ERROR: virtualenv not found.\nTrove development'
                    ' requires virtualenv, please install it using your'
                    ' favorite package management tool')
            print('done.')
    print('done.')


def create_virtualenv(venv=VENV):
    """Creates the virtual environment and installs PIP only into the
    virtual environment
    """
    print('Creating venv...'),
    run_command(['virtualenv', '-q', '--no-site-packages', VENV])
    print('done.')
    print('Installing pip in virtualenv...'),
    if not run_command(['tools/with_venv.sh', 'easy_install', 'pip']).strip():
        die("Failed to install pip.")
    print('done.')


def install_dependencies(venv=VENV):
    print('Installing dependencies with pip (this can take a while)...')
    # Install greenlet by hand - just listing it in the requires file does not
    # get it in stalled in the right order
    run_command(['tools/with_venv.sh', '-E', venv, 'pip', 'install',
                 'greenlet'], redirect_output=False)
    for requires in (PIP_REQUIRES, TEST_REQUIRES):
        run_command(['tools/with_venv.sh', '-E', venv, 'pip', 'install', '-r',
                     requires], redirect_output=False)

    # Tell the virtual env how to "import trove"
    pthfile = os.path.join(venv, "lib", PY_VERSION, "site-packages",
        "trove.pth")
    f = open(pthfile, 'w')
    f.write("%s\n" % ROOT)


def print_help():
    help = """
    Trove development environment setup is complete.

    Trove development uses virtualenv to track and manage Python
    dependencies while in development and testing.

    To activate the Trove virtualenv for the extent of your current shell
    session you can run:

    $ . .venv/bin/activate

    Or, if you prefer, you can run commands in the virtualenv on a case by case
    basis by running:

    $ tools/with_venv.sh <your command>

    Also, make test will automatically use the virtualenv.
    """
    print(help)


def main(argv):
    check_python_version()
    check_dependencies()
    create_virtualenv()
    install_dependencies()
    print_help()

if __name__ == '__main__':
    main(sys.argv)



# Copyright 2013 OpenStack Foundation
# Copyright 2013 Rackspace Hosting
# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import inspect
import os

from oslo_log import log as logging
import oslo_messaging as messaging
from oslo_service import loopingcall
from oslo_service import service
from oslo_utils import importutils
from osprofiler import profiler

from trove.common import cfg
from trove.common import profile
from trove.common.rpc import secure_serializer as ssz
from trove import rpc


CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class RpcService(service.Service):

    def __init__(self, key, host=None, binary=None, topic=None, manager=None,
                 rpc_api_version=None, secure_serializer=ssz.SecureSerializer):
        super(RpcService, self).__init__()
        self.key = key
        self.host = host or CONF.host
        self.binary = binary or os.path.basename(inspect.stack()[-1][1])
        self.topic = topic or self.binary.rpartition('trove-')[2]
        _manager = importutils.import_object(manager)
        self.manager_impl = profiler.trace_cls("rpc")(_manager)
        self.rpc_api_version = rpc_api_version or \
            self.manager_impl.RPC_API_VERSION
        self.secure_serializer = secure_serializer
        profile.setup_profiler(self.binary, self.host)

    def start(self):
        LOG.debug("Creating RPC server for service %s", self.topic)

        target = messaging.Target(topic=self.topic, server=self.host,
                                  version=self.rpc_api_version)

        if not hasattr(self.manager_impl, 'target'):
            self.manager_impl.target = target

        endpoints = [self.manager_impl]
        self.rpcserver = rpc.get_server(
            target, endpoints, key=self.key,
            secure_serializer=self.secure_serializer)
        self.rpcserver.start()

        # TODO(hub-cap): Currently the context is none... do we _need_ it here?
        report_interval = CONF.report_interval
        if report_interval > 0:
            pulse = loopingcall.FixedIntervalLoopingCall(
                self.manager_impl.run_periodic_tasks, context=None)
            pulse.start(interval=report_interval,
                        initial_delay=report_interval)
            pulse.wait()

    def stop(self):
        # Try to shut the connection down, but if we get any sort of
        # errors, go ahead and ignore them.. as we're shutting down anyway
        try:
            self.rpcserver.stop()
        except Exception:
            LOG.info("Failed to stop RPC server before shutdown. ")
            pass

        super(RpcService, self).stop()
